import java.util.List;
import java.util.Vector;
import java.io.IOException;
import java.io.DataInputStream;
import java.lang.StringBuffer;
import java.util.Iterator;

public class JsScanner
{
    DataInputStream _input;
    public static final int TT_KEYWORD = 1;
    public static final int TT_IDENTIFIER = 2;
    public static final int TT_BRACKET = 3;
    public static final int TT_OPERATOR = 4;
    public static final int TT_QUOTED = 5;
    public static final int TT_NUMBER = 6;
    public static final int TT_SEMICOLON = 7;

    private static final int STATE_START = 0;
    private static final int STATE_IDENTIFIER = 1;
    private static final int STATE_QUOTED = 2;
    private static final int STATE_QUOTED_ESCAPED = 3;
    private static final int STATE_NUMBER = 4;

    private String stateName(int state )
    {
        return state == STATE_START ? "START" :
            state == STATE_IDENTIFIER ? "IDENTITIFER" :
            state == STATE_QUOTED ? "QUOTED" :
            state == STATE_QUOTED_ESCAPED ? "QUOTED_ESCAPED" :
            state == STATE_NUMBER ? "NUMBER" :
            "UNKNOWN";
    }

    private int _state = STATE_START;
    private StringBuffer _identifier;
    private StringBuffer _constant;
    private char _quoteChar;
    private char _lastChar;
    private List _tokens;
    private Token _prev;

    private boolean _js = false;
    private boolean _css = false;

    private static final String keywords[] = {
        "break", "case", "catch", "continue",
        "Default", "delete", "do", "else",
        "finally", "for", "function", "if", "in",
        "instanceof", "new", "return", "switch", "this",
        "throw", "try", "typeof", "var", "void",
        "while", "with"
    };

    private static boolean debug = false;
    private void dbgprint( String str )
    {
        if ( debug ) {
            System.out.println( str );
        }
    }

    class Token {
        public Token( int type, String text )
        {
            this.type = type;
            this.text = text;
        }
        public String toString()
        {
            String mtype = 
                type == TT_KEYWORD ? "TT_KEYWORD" :
                type == TT_IDENTIFIER ? "TT_IDENTIFIER" :
                type == TT_BRACKET ? "TT_BRACKET" :
                type == TT_OPERATOR ? "TT_OPERATOR" :
                type == TT_QUOTED ? "TT_QUOTED" :
                type == TT_SEMICOLON ? "TT_SEMICOLON" :
                type == TT_NUMBER ? "TT_NUMBER" :
                "UNKOWN";

            return mtype + ": " + text;
        }
        int type;
        String text;
        int tagId = -1;
    };
    
    private Tags _tags;
    
    public JsScanner(Tags tags, String Type)
    {
        _tokens = new Vector();

        if ( Type.equals("js") ) {
            _js = true;
        } else if ( Type.equals("css" ) ) {
            _css = true;
        }

        _tags = tags;
    }

    public List parse(DataInputStream input)
    {
        _input = input;

        try {
            for ( ;; ) {
                int ret = input.read();
                if ( ret == -1 ) {
                    break;
                }

                parseCharacter( (char)ret );
                _lastChar = (char)ret;
            }
        } catch ( IOException e ) {
            System.out.println( e.toString() );
        }

        addTags();

        return _tokens;
    }

    private void parseCharacter( char ch )
    {
        dbgprint(stateName(_state) + ": \'" + ch + "\'");

        switch( _state ) {
            case STATE_START:
                start(ch);
                break;

            case STATE_IDENTIFIER:
                identifier(ch);
                break;

            case STATE_QUOTED:
                quoted(ch);
                break;

            case STATE_QUOTED_ESCAPED:
                quoted_escaped(ch);
                break;

            case STATE_NUMBER:
                number(ch);
                break;
        }
    }

    private void start( char ch ) {
        if ( ch == '(' || ch == ')' || ch == '[' || ch == ']' || ch == '{' ||
                ch == '}' ) 
        {
            output(ch, TT_BRACKET );
        } else if ( ch == '?' || ch == '+' || ch == '-' || ch == '.' || ch == ':' ||
                ch == ',' || ch == '*' || ch == '/' || ch == '=' || ch == '!' ||
                ch == '>' || ch == '<' || ch == '|' || ch == '&' || ch == '^' ) 
        {
            output( ch, TT_OPERATOR );
        } else if ( ch == ';' ) {
            output( ch, TT_SEMICOLON );
        } else if ( Character.isLetter(ch) || ch == '_' ) {
            switchState( STATE_IDENTIFIER );
            parseCharacter(ch);
        } else if ( ch == '\'' || ch == '\"' ) {
            if ( _lastChar == '\\' ) {
                // Handle \" inside regular expressions.
                output( ch, TT_OPERATOR );
            } else {
                _quoteChar = ch;
                switchState( STATE_QUOTED );
                _constant.append( ch );
            }
        } else if ( Character.isDigit( ch ) ) {
            switchState( STATE_NUMBER );
            parseCharacter( ch );
        }
    }

    private void switchState( int state ) 
    {
        _state = state;
        switch( _state ) {
            case STATE_IDENTIFIER:
                _identifier = new StringBuffer();
                break;

            case STATE_NUMBER:
            case STATE_QUOTED:
                _constant = new StringBuffer();
                break;

            case STATE_QUOTED_ESCAPED:
                break;
        }
    }

    private void identifier( char ch ) 
    {
        if ( Character.isLetterOrDigit(ch) || ch == '_' ||
                _css && ch == '-') {
            _identifier.append( ch );
        } else {
            output(_identifier.toString(), TT_IDENTIFIER );
            switchState( STATE_START );
            parseCharacter( ch );
        }
    }

    private void quoted( char ch )
    {
        _constant.append( ch );

        if ( ch == '\\' ) {
            switchState( STATE_QUOTED_ESCAPED );
        } else if ( ch == _quoteChar ) {
            output( _constant.toString(), TT_QUOTED );
            switchState( STATE_START );
        }
    }

    private void quoted_escaped( char ch )
    {
        _constant.append(ch);
        switchState( STATE_QUOTED );
    }

    private void number( char ch )
    {
        if ( Character.isDigit( ch ) || ch == '.' ) {
            _constant.append(ch);
        } else {
            output( _constant.toString(), TT_NUMBER );
            switchState( STATE_START );
            parseCharacter( ch );
        }
    }

    private void output( char ch, int type )
    {
        char array[] = new char[1];
        array[0] = ch;
        output( new String(array), type );
    }

    private void output( String text, int type )
    {
        if ( type == TT_IDENTIFIER ) {
            // possibly change to keyword.
            for( int i = 0; i < keywords.length; i++ ) {
                if ( keywords[i].equals(text) ) {
                    type = TT_KEYWORD;
                    break;
                }
            }

        }

        if ( type == TT_OPERATOR ) {
            if ( _prev != null ) {
                if ( _prev.type == TT_OPERATOR ) {
                    _prev.text = _prev.text + text;
                    return;
                }
            }
        }
        
        Token token = new Token(type, text );
        _tokens.add( token );
        _prev = token;
    }

    private void addTags()
    {
        // for each tag,
        Iterator iter = _tokens.iterator();
        Token prev = null;
        while( iter.hasNext() ) {
            Token cur = (Token)iter.next();
            
            // if the previous tag was "function", and the current tag is an
            // identifier, add the tag.
            if ( prev != null && 
                    prev.text.equals("function") &&
                    cur.type == TT_IDENTIFIER )
            {
                cur.tagId = _tags.add( cur.text, "js" );
            }

            prev = cur;
        }
    }
}
