import java.io.*;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import java.net.URL;

class Main {

    private static Tags _tags;

    public static void main(String[] args) 
    {
        DataInputStream input;
        PrettyOutput output;
        String type;

        Options.addLongValue("output", "text");
        Options.setOtherArgsCount( 0, 1 );
        if ( !Options.parse( args ) ) {
            return;
        }

        type = Options.getString( "output" );
        if ( type.equals( "text" ) ) {
            output = new PrettyOutput();
        } else if ( type.equals( "html" ) ) { 
            output = new HtmlOutput();
        } else {
            System.out.println("Not understood: " + type );
            return;
        }

        String[] otherArgs = Options.getOtherArgs();
        if ( otherArgs.length == 0 ) {
            input = new DataInputStream(System.in);
        } else {
            try {
                input = openFile( otherArgs[0] );
            } catch ( IOException e ) {
                System.out.println("Couldn't open " + otherArgs[0]);
                System.out.println( e.getMessage() );
                return;
            }
        }

        _tags = new Tags();

        if ( otherArgs[0].endsWith(".js") ) {
            printScriptFile( "js", input, output );
            return;
        } else if ( otherArgs[0].endsWith(".css") ) {
            printScriptFile( "css", input, output );
            return;
        } else {
            // html
            printHtmlFile( input, output );
        }
    }

    private static void printScriptFile(String type, DataInputStream input, 
            PrettyOutput output )
    {
        JsScanner scanner = new JsScanner(_tags, type);
        System.out.println("SCript type: " + type );

        List<HtmlElement> list = new Vector<HtmlElement>();
        list.add( new JsElement( scanner.parse(input), type ) );
        
        PrettyPrinter printer = new PrettyPrinter(output, _tags);
        printer.prettyPrint(list);
    }

    private static void printHtmlFile( DataInputStream input, PrettyOutput output )
    {
        try {
            HtmlScanner scanner = new HtmlScanner(_tags);
            scanner.parse( input );
            List elements = scanner.getElements();
            HtmlIndenter indenter = new HtmlIndenter();
            indenter.indent(elements);

            PrettyPrinter printer = new PrettyPrinter(output, _tags);
            printer.prettyPrint(elements);
                
        } catch(Exception e) {
            System.err.println("exception: "+e);
            e.printStackTrace();
        }
    }

    private static DataInputStream openFile( String filename ) 
        throws IOException
    {
        InputStream file;

        if ( filename.startsWith( "http:" ) ) {
            URL url = new URL(filename);
            file = url.openStream();
        } else {
            file = new FileInputStream( filename );
        }

        return new DataInputStream(file);
    }
}
