import java.util.TreeMap;

class Options
{
    private Options() {
        
    }

    public static void addLongFlag( String name ) {
        _options.put( name, new Integer( TYPE_LONG_BOOLEAN ) );
    }
    
    public static void addLongValue( String name, String Default ) {
        _options.put( name, new Integer( TYPE_LONG_STRING ) );
        _values.put( name, Default );
    }

    public static void setOtherArgsCount( int min, int max ) {
        _other = new String[max];
    }

    public static String[] getOtherArgs() {
        return _other;
    }

    public static boolean parse( String argv[] )
    {
        try {
            for( int i = 0; i < argv.length; i++ ) {
                if ( argv[i].startsWith( "--" ) ) {
                    String name = argv[i].substring( 2 );
                    Integer type = (Integer)_options.get( name );
                    if ( type == null ) {
                        throw new Exception("Invalid option: " + argv[i] );
                    }

                    if ( type.equals( TYPE_LONG_BOOLEAN ) ) {
                        _values.put( name, new Boolean( true ) );
                    } else if ( type.equals( TYPE_LONG_STRING ) ) {
                        if ( i == argv.length - 1 ) {
                            throw new Exception("Option " + argv[i] + 
                                    " requires an argument.");
                        } else {
                            _values.put( name, argv[++i] );
                        }
                    }
                } else if ( _otherCount == _other.length ) {
                    throw new Exception("Too many arguments specified.");
                } else {
                    _other[_otherCount++] = argv[i];
                }
            }
        } catch( Exception e ) {
            System.out.println( e.getMessage() );
            return false;
        }

        if ( _otherCount < _otherMax ) {
            String[] array = new String[_otherCount];
            System.arraycopy(_other, 0, array, 0, _otherCount);
            _other = array;
        }

        return true;
    }

    public static boolean getBoolean( String flagName ) 
    {
        Object obj = _values.get( flagName );
        if ( obj != null && obj instanceof Boolean ) {
            return true;
        }

        return false;
    }

    public static String getString( String name )
    {
        Object obj = _values.get( name );
        if ( obj != null && obj instanceof String ) {
            return (String) obj;
        }

        return null;
    }

    private static final int TYPE_LONG_BOOLEAN = 1;
    private static final int TYPE_LONG_STRING = 2;

    private static TreeMap<String, Integer> _options = new TreeMap<String, Integer>();
    private static TreeMap _values = new TreeMap();
    private static int _otherMax;
    private static int _otherMin;
    private static int _otherCount;
    private static String[] _other = new String[0];
    private static Options _ref;
}
