import java.util.List;
import java.util.Iterator;
import java.util.Vector;

public class PrettyPrinter
{
    PrettyOutput output;
    Tags _tags;
    
    public PrettyPrinter(PrettyOutput output, Tags tags)
    {
        _tags = tags;
        this.output = output;
    }

    public void prettyPrint(List elements)
    {
        Iterator iter = elements.iterator();
        while( iter.hasNext() ) {
            HtmlElement elem = (HtmlElement)iter.next();
            output.setLevel(elem.level);

            if ( elem instanceof HtmlTag ) {
                print((HtmlTag)elem);
            } else if ( elem instanceof HtmlText ) {
                print((HtmlText)elem);
            } else if ( elem instanceof JsElement ) {
                print((JsElement)elem);
            }
        }
    }


    private void print(HtmlTag tag)
    {
        output.newline();
        output.write("<", output.HTML_PUNC);
        output.write(tag.name.toString(), output.HTML_TAG_NAME);
        Iterator iter = tag.attributes.iterator();
        while( iter.hasNext() ) {
            HtmlTag.Attribute attribute = (HtmlTag.Attribute)iter.next();
            output.checkWrap();

            output.write(" ", output.SAME_STYLE);
            output.write(attribute.name, output.HTML_ATTRIBUTE_NAME);
            if ( attribute.value != null ) {
                output.write("=", output.HTML_PUNC);
                if ( attribute.value.indexOf('\"') != -1 ) {
                    output.write( "\'" + attribute.value + "\'", output.HTML_ATTRIBUTE_VALUE );
                } else {
                    output.write( "\"" + attribute.value + "\"", output.HTML_ATTRIBUTE_VALUE );
                }
            }
        }
        output.write(">", output.HTML_PUNC);
    }

    private void print(HtmlText text)
    {
        output.newline();
        String words[] = text.text.trim().split("\\s");
        for( int i = 0; i < words.length; i++ ) {
            output.write( words[i] + " ", output.HTML_TEXT );
            output.checkWrap();
        }
    }

    private void print(JsElement js)
    {
        if ( js.type == "css" ) {
            printCss( js );
        } else if ( js.type == "js" ) {
            printJs( js );
        }

    }

    private void printJs( JsElement js ) 
    {
        int level = js.level;
        output.newline();
        Iterator iter = js.tokens.iterator();
        boolean first = true;
        boolean lastWasEndBrace = false;
        while( iter.hasNext() ) {
            JsScanner.Token token = (JsScanner.Token)iter.next();
            boolean wasEndBrace = lastWasEndBrace;
            String style = output.HTML_TEXT;
            int tagId = -1;
            lastWasEndBrace = false;

            if ( token.type == JsScanner.TT_KEYWORD ) {
                style = output.JS_KEYWORD;
            } else if ( token.type == JsScanner.TT_QUOTED ) {
                style = output.JS_CONSTANT;
            } else if ( token.type == JsScanner.TT_NUMBER ) {
                style = output.JS_CONSTANT;
            } else {
                if ( token.tagId != -1 ) {
                    tagId = token.tagId;
                    style = output.JS_FUNCTION;
                } else {
                    tagId = _tags.lookup( token.text, "js" );
                    if ( tagId != -1 ) {
                        style = output.JS_CALL;
                    }
                }
            }

            if ( token.text.equals("{") ) {
                output.newline();
                output.write(token.text, style, tagId);
                output.setLevel(++level);
                output.newline();
                first = true;
                continue;
            } else if ( token.text.equals("}")) {
                output.setLevel(--level);
                output.newline();
                output.write(token.text, style, tagId);
                output.newline();
                first = true;
                lastWasEndBrace = true;
                continue;
            } else if ( token.type == JsScanner.TT_OPERATOR ) {
                    if ( token.text.equals(",") ) {
                        output.write(token.text, style, tagId);
                    } else if ( token.text.equals(".") ) {
                        output.write(token.text, style, tagId);
                        first = true;
                    } else {
                        output.write(" " + token.text, style, tagId);
                    }
                continue;
            } else if ( token.type == JsScanner.TT_BRACKET ) {
                output.write(token.text, style, tagId);
                first = true;
            } else if ( token.type == JsScanner.TT_SEMICOLON ) {
                output.write(token.text, style, tagId);
            } else {
                if ( wasEndBrace && !token.text.equals("else") ) {
                    output.newline();
                    first = true;
                }
                
                if ( first ) {
                    output.write(token.text, style, tagId);
                    first = false;
                } else {
                    output.write(" " + token.text, style, tagId);
                }
            }

            if ( token.type == JsScanner.TT_SEMICOLON ) {
                output.newline();
                first = true;
            } 
        }
    }

    private void printCss( JsElement css ) 
    {
        int level = css.level;
        output.newline();
        Iterator iter = css.tokens.iterator();
        boolean first = true;
        boolean lastWasEndBrace = false;

        String prev = "";
        
        while( iter.hasNext() ) {
            JsScanner.Token token = (JsScanner.Token)iter.next();
            String style = output.HTML_TEXT;
            if ( !first && !token.text.equals(";") && !token.text.equals("(") &&
                    !token.text.equals(";") && !token.text.equals(":") && 
                    !token.text.equals(".") && !token.text.equals(",") &&
                    !token.text.equals("px") && !token.text.equals("em") &&
                    !token.text.equals("in") && !token.text.equals("in") &&
                    !token.text.equals("pt") && 
                    !prev.equals(".") && !prev.equals(":") ) {
                output.write(" ", style);
            }

            if ( token.text.equals("}") ) {
                output.setLevel(--level);
                output.newline();
            }

            output.write(token.text, style);

            first = false;

            if ( token.text.equals("{") ) {
                output.setLevel(++level);
                output.newline();
                first = true;
            } else if ( token.text.equals("}") ) {
                output.newline();
                output.newline();
                first = true;
            } else if ( token.text.equals(";") ) {
                output.newline();
                first = true;
            }

            prev = token.text;
        }
    }
}
